#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------


class ZorroTool

def initialize
    @ip1 = nil
    @ip2 = nil
    @xdown = 0
    @ydown = 0
	@x1=0
	@x2=0
	@y1=0
	@y2=0
end

def activate
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false    
    self.reset(nil)
end

def deactivate(view)
    view.invalidate if @drawn
end

def onMouseMove(flags, x, y, view)
    if( @state == 0 )
       
        @ip.pick view, x, y
        if( @ip != @ip1 )
          
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
		@x1=x
		@y1=y
            view.tooltip = @ip1.tooltip
        end
    else
       
        @ip2.pick view, x, y
        view.tooltip = @ip2.tooltip if( @ip2.valid? )
        view.invalidate
        
       
        if( @ip2.valid? )
            length = @ip1.position.distance(@ip2.position)
            Sketchup::set_status_text length.to_s, SB_VCB_VALUE
		@x2=x
		@y2=y
        end
        
      
        if( (x-@xdown).abs > 10 || (y-@ydown).abs > 10 )
            @dragging = true
        end
    end
end

# The onLButtonDOwn method is called when the user presses the left mouse button.
def onLButtonDown(flags, x, y, view)
    # When the user clicks the first time, we switch to getting the
    # second point.  When they click a second time we create the line
    if( @state == 0 )
        @ip1.pick view, x, y
        if( @ip1.valid? )
            @state = 1
            Sketchup::set_status_text $exStrings.GetString("Draw cut line"), SB_PROMPT
            @xdown = x
            @ydown = y
		@x1=x
		@y1=y
        end
    else
        # create the line on the second click
        if( @ip2.valid? )
            self.save_region(view)
            self.reset(view)
        end
    end
    
    # Clear any inference lock
    view.lock_inference
end

# The onLButtonUp method is called when the user releases the left mouse button.
def onLButtonUp(flags, x, y, view)
    # If we are doing a drag, then create the line on the mouse up event
    if( @dragging && @ip2.valid? )
        self.save_region(view)
        self.reset(view)
    end
end

def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
        @shift_down_time = Time.now
        
        # if we already have an inference lock, then unlock it
        if( view.inference_locked? )
            # calling lock_inference with no arguments actually unlocks
            view.lock_inference
        elsif( @state == 0 && @ip1.valid? )
            view.lock_inference @ip1
        elsif( @state == 1 && @ip2.valid? )
            view.lock_inference @ip2, @ip1
        end
    end
end


def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
        view.inference_locked? &&
        (Time.now - @shift_down_time) > 0.5 )
        view.lock_inference
    end
end


def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
        
        if( @ip2.valid? )
            @ip2.draw(view) if( @ip2.display? )
     
            view.set_color_from_line(@ip1, @ip2)
            self.draw_geometry([@x1,@y1,0],[@x2,@y1,0],[@x2,@y2,0],[@x1,@y2,0], view)
            @drawn = true
        end
    end
end

# onCancel is called when the user hits the escape key
def onCancel(flag, view)
    self.reset(view)
end


# The following methods are not directly called from SketchUp.  They are
# internal methods that are used to support the other methods in this class.

# Reset the tool back to its initial state
def reset(view)
    # This variable keeps track of which point we are currently getting
    @state = 0
	@x1=0
	@x2=0
	@y1=0
	@y2=0
    
    # Display a prompt on the status bar
    Sketchup::set_status_text($exStrings.GetString("Draw cut line"), SB_PROMPT)
    
    # clear the InputPoints
    @ip1.clear
    @ip2.clear
    
    if( view )
        view.tooltip = nil
        view.invalidate if @drawn
    end
    
    @drawn = false
    @dragging = false
end

# 
def save_region(view)

ray1=view.pickray(@x1,@y1)
ray2=view.pickray(@x2,@y2)
v1=ray1[1]
v2=ray2[1]
eye=ray1[0]
trans=Geom::Transformation.scaling(1000000)
normal=v1.cross v2
p1=Geom::Point3d.new(v1[0],v1[1],v1[2])
p2=Geom::Point3d.new(v2[0],v2[1],v2[2])
p1.transform! trans
p2.transform! trans
Sketchup.active_model.start_operation("Knife")
ents=Sketchup.active_model.active_entities
knife_group=ents.add_group
knife_ents=knife_group.entities
sel=Sketchup.active_model.selection
knife=knife_ents.add_face(eye,p1,p2)
ents.intersect_with(false,Geom::Transformation.new,ents,knife_group.transformation,false,[knife_group])
ents.erase_entities(knife_group)
Sketchup.active_model.commit_operation
	
end

# Draw the geometry
def draw_geometry(pt1, pt2, pt3, pt4, view)
    view.draw2d(GL_LINES,[pt1,pt3])
	#pt1=Geom::Point3d.new(0,0,0)
	#pt2=Geom::Point3d.new(0,100,0)
	#pt3=Geom::Point3d.new(100,100,0)
	#pt4=Geom::Point3d.new(100,0,0)
	#view.draw_points([pt1,pt3,pt2,pt4],25,2,"red")
end

end #class


#-----------------------------------------------------------------------------
# This functions is just a shortcut for selecting the new tool
def zorrotool
    Sketchup.active_model.select_tool ZorroTool.new
end

if( not file_loaded?("Zorro.rb") )
main_menu = UI.menu("Tools")

main_menu.add_item("Zorro") {(zorrotool)}
end
file_loaded("Zorro.rb")
